import { Plugin } from "vite";
import { defineConfig } from "vite";

const esmContentScriptLoader = (): Plugin => ({
  name: "esm-content-script-loader",
  generateBundle(_options, bundle, _isWrite) {
    // Add the loader to the bundle before the bundle is written to the disk
    bundle["content-script-loader.js"] = {
      exports: [],
      facadeModuleId: null,
      isDynamicEntry: false,
      isEntry: true,
      isImplicitEntry: false,
      moduleIds: [],
      name: "content-script-loader",
      type: "chunk",
      dynamicImports: [],
      fileName: "content-script-loader.js",
      implicitlyLoadedBefore: [],
      importedBindings: {},
      imports: [],
      modules: {},
      map: null,
      sourcemapFileName: null,
      referencedFiles: [],
      viteMetadata: undefined,
      code: `
(async () => {
  console.log("Importing 'content-script'...")
  await import(
    /* vite-ignore */
    chrome.runtime.getURL('/content-script.js')
  )
  console.log("Imported 'content-script'!")
})()
      `.trim(),
      preliminaryFileName: "content-script-loader.js",
    };
  },
});

export default defineConfig({
  build: {
    rollupOptions: {
      input: {
        popup: "src/popup.html",
        "content-script": "src/content-script.ts",
        background: "src/background.ts",
      },
      output: {
        format: "esm",
        entryFileNames: "[name].js",
      },
    },
    // Not necessary, just for clearity when looking at output files
    minify: false,
  },
  plugins: [esmContentScriptLoader()],
});
